/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#include "perceptuum2.rc.h"

#include "AboutDialog.h"   /// own header is included last


using namespace perceptuum2::platform_ui;




/// standard object services -----------------------------------------------------------------------
AboutDialog::AboutDialog
(
	HINSTANCE   hInstance,
	HWND        hWndParent,
	HDC         dc,
	const char* pVersion,
	const char* pBuildTime
)
 :	Win32Dialog( hInstance, hWndParent ),
	hImage_m( 0 ),
	dc_m( 0 ),
	defaultBitmap_m( 0 ),
	version_m  ( pVersion ),
	buildTime_m( pBuildTime ),
	copyright_m(),
	uri_m      ()
{
	/// load image resource
	hImage_m = ::LoadBitmap( hInstance_m, MAKEINTRESOURCE( IDB_TITLE_IMAGE ) );
	if( hImage_m != 0 )
	{
		/// make DC
		dc_m = ::CreateCompatibleDC( dc );
		if( dc_m != 0 )
		{
			/// put bitmap into DC
			defaultBitmap_m = (HBITMAP)::SelectObject( dc_m, hImage_m );
			if( defaultBitmap_m == 0 )
			{
				::DeleteDC( dc_m );
				::DeleteObject( hImage_m );
				dc_m     = 0;
				hImage_m = 0;
			}
		}
		else
		{
			::DeleteObject( hImage_m );
			hImage_m = 0;
		}
	}

	/// load info strings resources
	{
		TCHAR buf[32];
		bool  ok;// = false;

		ok = bool(::LoadString( hInstance_m, IDS_APP_VERSION_LABEL, (LPSTR)&buf, sizeof(buf)/sizeof(TCHAR) ));
		if( ok )
		{
			version_m = ((string(buf) += ' ') += version_m);
		}

		ok = bool(::LoadString( hInstance_m, IDS_APP_DESC1, (LPSTR)&buf, sizeof(buf)/sizeof(TCHAR) ));
		if( ok )
		{
			version_m = ((string(buf) += ", ") += version_m);
		}

		ok = bool(::LoadString( hInstance_m, IDS_APP_NAME, (LPSTR)&buf, sizeof(buf)/sizeof(TCHAR) ));
		if( ok )
		{
			version_m = ((string(buf) += ' ') += version_m);
		}

		ok = bool(::LoadString( hInstance_m, IDS_APP_BUILT_LABEL, (LPSTR)&buf, sizeof(buf)/sizeof(TCHAR) ));
		if( ok )
		{
			buildTime_m = ((string(buf) += ' ') += buildTime_m);
		}

		ok = bool(::LoadString( hInstance_m, IDS_COPYRIGHT, (LPSTR)&buf, sizeof(buf)/sizeof(TCHAR) ));
		if( ok )
		{
			copyright_m = buf;
		}

		ok = bool(::LoadString( hInstance_m, IDS_URI, (LPSTR)&buf, sizeof(buf)/sizeof(TCHAR) ));
		if( ok )
		{
			uri_m = buf;
		}
	}
}


AboutDialog::~AboutDialog()
{
	::SelectObject( dc_m, defaultBitmap_m );
	::DeleteObject( hImage_m );
	::DeleteDC( dc_m );
}




/// commands ---------------------------------------------------------------------------------------
int AboutDialog::run()
{
	return Win32Dialog::run();

//	int dlgReturn;
//
//	/// check if image loaded
//	if( hImage_m != NULL )
//	{
//		/// do dialog
//		dlgReturn = Win32Dialog::run();
//	}
//	else
//	{
//		/// do message box
//		TCHAR buf1[64];
//		const int c1 = ::LoadString( hInstance_m, IDS_ABOUT_MESSAGE, (LPSTR)&buf1, sizeof(buf1)/sizeof(TCHAR) );
//		TCHAR buf2[16];
//		const int c2 = ::LoadString( hInstance_m, IDS_ABOUT_TITLE,   (LPSTR)&buf2, sizeof(buf2)/sizeof(TCHAR) );
//
//		if( (c1 != 0) && (c2 != 0) )
//		{
//			::MessageBox( hWndParent_m, buf1, buf2, MB_OK | MB_ICONINFORMATION );
//		}
//		dlgReturn = FALSE;
//	}
//
//	return dlgReturn;
}




/// overrides --------------------------------------------------------------------------------------
int AboutDialog::getDialogResourceId() const
{
	return IDD_ABOUTBOX;
}


bool AboutDialog::dialogProc
(
	HWND   hWndDlg,
	UINT   message,
	WPARAM wParam,
	LPARAM //lParam
)
{
	static const dword IMG_WIDTH     = 400;
	static const dword IMG_HEIGHT    = 248;
	static const dword LETTER_HEIGHT = 12;
	static const dword LETTER_LEFT   = LETTER_HEIGHT * 2;
	/// if these are > 0, theres a gap between the image and the window frame,
	/// and the image isnt repainted properly !??
	static const dword BORDER        = 0;//2;
	static const dword FRAME         = 0;//2;    /// should always be 2

	static const dword TXT_HEIGHT        = LETTER_HEIGHT * 5;
	static const dword WIDTH             = IMG_WIDTH  + (BORDER + FRAME) * 2;
	static const dword HEIGHT            = IMG_HEIGHT + TXT_HEIGHT + (BORDER + FRAME) * 4;
	static const dword TXT_TOP           = IMG_HEIGHT + (BORDER + FRAME) * 2;
	static const dword LETTER_FIRST_LINE = TXT_TOP + (BORDER + FRAME) + (LETTER_HEIGHT / 2) + 2;


	bool didHandleMessage = true;

	switch( message )
	{
	case WM_INITDIALOG :
		{
			bool ok = true;

			///// get image width height
			//BITMAP bm;
			//ok &= bool( ::GetObject( hImage_m, sizeof(bm), &bm ) );

			/// get desktop width height
			HWND dw = ::GetDesktopWindow();
			RECT screen;
			ok &= bool( ::GetClientRect( dw, &screen ) );

			/// get window border size
			RECT cr;
			RECT wr;
			ok &= bool( ::GetClientRect( hWndDlg, &cr ) );
			ok &= bool( ::GetWindowRect( hWndDlg, &wr ) );
			const POINT border = { (wr.right - wr.left) - cr.right,  (wr.bottom - wr.top) - cr.bottom };

			/// set width height and position at center screen
			if( ok )
			{
				const POINT pos  = { (screen.right - WIDTH - border.x) / 2,  (screen.bottom - HEIGHT - border.y) / 2 };
				const POINT size = { WIDTH + border.x, HEIGHT + border.y };
				ok &= bool( ::MoveWindow( hWndDlg, pos.x, pos.y, size.x, size.y, FALSE ) );
			}

			if( !ok )
			{
				::EndDialog( hWndDlg, FALSE );
			}

			break;
		}
	case WM_PAINT :
		{
			PAINTSTRUCT ps;
			HDC paintDc = ::BeginPaint( hWndDlg, &ps );

			/// draw frames
			{
				static const dword FRAME_LEFT  = BORDER;
				static const dword FRAME_RIGHT = WIDTH - BORDER;

				//RECT imgFrame = { FRAME_LEFT, BORDER, FRAME_RIGHT, TXT_TOP - BORDER };
				//::DrawEdge( paintDc, &imgFrame, EDGE_ETCHED, BF_RECT );

				RECT txtFrame = { FRAME_LEFT, TXT_TOP + BORDER, FRAME_RIGHT, HEIGHT - BORDER };
				::DrawEdge( paintDc, &txtFrame, EDGE_ETCHED, BF_RECT );

				HBRUSH whiteBrush = static_cast<HBRUSH>( ::GetStockObject( WHITE_BRUSH ) );
				//RECT txtGround = { FRAME_LEFT + FRAME, TXT_TOP + BORDER + FRAME, FRAME_RIGHT - FRAME, HEIGHT - BORDER - FRAME };
				RECT txtGround = { FRAME_LEFT + 2, TXT_TOP + BORDER + 2, FRAME_RIGHT - 2, HEIGHT - BORDER - 2 };
				::FillRect( paintDc, &txtGround, whiteBrush );
			}

			/// draw logo image
			::BitBlt( paintDc,
			          ps.rcPaint.left + (BORDER + FRAME), ps.rcPaint.top + (BORDER + FRAME),
			          ps.rcPaint.right - ps.rcPaint.left, ps.rcPaint.bottom - ps.rcPaint.top,
					  dc_m,
			          ps.rcPaint.left, ps.rcPaint.top,
					  SRCCOPY );

			/// draw text
			{
				HFONT font = ::CreateFont( LETTER_HEIGHT, 0, 0, 0, 0,
										   false, false, false,
										   DEFAULT_CHARSET, OUT_TT_PRECIS, CLIP_DEFAULT_PRECIS, PROOF_QUALITY,
										   VARIABLE_PITCH | FF_SWISS | 0x04, "verdana" );
										   //FIXED_PITCH | FF_MODERN | 0x04, "lucida console" );
				HFONT dfltFont = static_cast<HFONT>( ::SelectObject( paintDc, font ) );
				::SetBkColor( paintDc, COLORREF( 0x00FFFFFF ) );
				::SetTextColor( paintDc, COLORREF( 0x00000000 ) );

				dword linePos = LETTER_FIRST_LINE;
				::TextOut( paintDc, LETTER_LEFT, linePos, version_m.data(), DWORD(version_m.length()) );
				::TextOut( paintDc, LETTER_LEFT, linePos += LETTER_HEIGHT, buildTime_m.data(), DWORD(buildTime_m.length()) );
				::TextOut( paintDc, LETTER_LEFT, linePos += LETTER_HEIGHT + (LETTER_HEIGHT / 2), copyright_m.data(), DWORD(copyright_m.length()) );
				::TextOut( paintDc, LETTER_LEFT + (WIDTH / 2), linePos, uri_m.data(), DWORD(uri_m.length()) );

				::DeleteObject( ::SelectObject( paintDc, dfltFont ) );
			}

			::EndPaint( hWndDlg, &ps );

			break;
		}
	case WM_COMMAND :
		{
			if( wParam == IDCANCEL )
			{
				::EndDialog( hWndDlg, TRUE );
				break;
			}
		}
	case WM_LBUTTONDOWN :
		{
			::EndDialog( hWndDlg, TRUE );
			break;
		}
	default:
		didHandleMessage = false;
	}

	return didHandleMessage;
}
