/*--------------------------------------------------------------------

   Perceptuum renderer, version 2.1
   Copyright (c) 2002-2003,  Harrison Ainsworth.

   www.hxa7241.org

--------------------------------------------------------------------*/

/*--------------------------------------------------------------------

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License as
   published by the Free Software Foundation; either version 2 of the
   License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston,
   MA 02111-1307, USA

--------------------------------------------------------------------*/


#ifndef ObjectInstance_h
#define ObjectInstance_h


#include "Primitives.h"
#include "ObjectRenderable.h"
#include "Transplace.h"
#include <vector>
#include <deque>

#include "hxa7241_utility.h"




#include "perceptuum2_modelling.h"
namespace perceptuum2 { namespace modelling {
	using namespace hxa7241::primitives;
	using hxa7241::graphics::Transplace;
	using hxa7241::utility::SerializerOut;
	using hxa7241::utility::SerializerIn;
	using std::vector;
	using std::deque;


class ObjectInstance
	: public ObjectRenderable
{
/// standard object services -----------------------------------------------------------------------
public:
	         ObjectInstance( const Scene*, SerializerIn&, bool );
protected:
	         ObjectInstance( const Scene* );
public:
	virtual ~ObjectInstance();
	         ObjectInstance( const ObjectInstance& );
	ObjectInstance& operator=( const ObjectInstance& );


/// commands ---------------------------------------------------------------------------------------
public:
	virtual bool  update( float time );


/// queries ----------------------------------------------------------------------------------------
public:
	virtual bool    getEmitted( const Vector3f& posRands,
	                            vector<float>&  compositionStorage,
	                            Intersection&,
	                            Qualities& )                                                  const;

	virtual bool    getIntersection( const ObjectRenderableId&,
	                                 const Ray&,
	                                 float& distance,
	                                 Intersection& )                                          const;
	virtual void    getObjectsForProjection( const Transclip& transform,
	                                         ClipFlags        clipFlags,
	                                         vector<ProjectableObjectRef>& )                  const;
	virtual void    getMesh( Mesh&, const Mesh** )                                            const;

	virtual void    getQualities( const Intersection&, Qualities& )                           const;

	virtual Float01 getTransparency( const ObjectRenderableId&,
	                                 const Vector3f& position )                               const;
	virtual float   getRefractiveIndex( const ObjectRenderableId&,
	                                    const Vector3f& position )                            const;
	virtual const CieXyz& getEmittance( const ObjectRenderableId& )                           const;

	virtual const char* getClassName()                                                        const;


/// serialization ----------------------------------------------------------------------------------
public:
	friend  SerializerOut& operator<<( SerializerOut&, const ObjectInstance& );
	friend  SerializerIn&  operator>>( SerializerIn&,        ObjectInstance& );
	static const char* CLASS_NAME();
	static const char* CLASS_DESCRIPTION();


/// implementation ---------------------------------------------------------------------------------
	class SubPart
	{
	/// standard object services
	public:
		 SubPart() {}
		~SubPart() {}
		 SubPart( const SubPart& ) {}
		 SubPart& operator=( const SubPart& ) { return *this; }

	/// commands
		void        setObjectIndex( const dword index ) { objectIndex_m = index; }
		Transplace& getTransform()                      { return transform_m; }

	/// queries
		dword             getObjectIndex() const { return objectIndex_m; }
		const Transplace& getTransform()   const { return transform_m; }

		bool  getIntersection( dword                     subpartIndex,
		                       const Scene&              scene,
		                       const ObjectRenderableId& objectId,
		                       const Ray&                ray,
		                       Intersection&             intersection,
		                       float&                    distance )                           const;

	/// serialization
		friend  SerializerOut& output( SerializerOut&, const SubPart&, const Scene& );
		friend  SerializerIn&  input ( SerializerIn&,        SubPart&, const Scene& );
		static const char* CLASS_NAME();
		static const char* CLASS_DESCRIPTION();

	/// fields
	private:
		dword      objectIndex_m;
		Transplace transform_m;
		//dword      materialId_m;
	};
	friend  SerializerOut& output( SerializerOut&, const SubPart&, const Scene& );
	friend  SerializerIn&  input ( SerializerIn&,        SubPart&, const Scene& );

protected:
	virtual void  assign( const ObjectRenderable& );
	virtual void  assign( const ObjectInstance& );

	virtual void  setBase();
	virtual deque<SubPart>& subParts();
	        float getUpdateTime()                                                             const;
	        bool  getUpdateIsChanging()                                                       const;
	        void  setUpdateNote( float time, bool isChanging );

	virtual void  getSubObject( const ObjectRenderableId&,
	                            ObjectRenderableId&, const Transplace*& )                     const;


/// fields -----------------------------------------------------------------------------------------
private:
	deque<SubPart> subParts_m;
	float          previousUpdateTime_m;
	bool           previousIsChanging_m;

//	vector<dword>      subParts_m;
//	vector<Transplace> transforms_m;
////	vector<dword>      materialIds_m;

	static const char DESERIALIZATION_WARNING_1[];
};




/// serialization ----------------------------------------------------------------------------------
SerializerOut& operator<<( SerializerOut&, const ObjectInstance& );
SerializerIn&  operator>>( SerializerIn&,        ObjectInstance& );


SerializerOut& output( SerializerOut&, const ObjectInstance::SubPart&, const Scene& );
SerializerIn&  input ( SerializerIn&,        ObjectInstance::SubPart&, const Scene& );


}}




#endif//ObjectInstance_h
